# Importing packages
import csv
from datetime import datetime
import glob
import math
import numpy as np
import os
import pandas as pd
import random

# Retrieved from https://towardsdatascience.com/cronbachs-alpha-theory-and-application-in-python-d2915dd63586
def cronbach_alpha(df):
    # 1. Transform the df into a correlation matrix
    df_corr = df.corr()
    
    # 2.1 Calculate N
    # The number of variables equals the number of columns in the df
    N = df.shape[1]
    
    # 2.2 Calculate R
    # For this, we'll loop through the columns and append every
    # relevant correlation to an array calles "r_s". Then, we'll
    # calculate the mean of "r_s"
    rs = np.array([])
    for i, col in enumerate(df_corr.columns):
        sum_ = df_corr[col][i+1:].values
        rs = np.append(sum_, rs)
    mean_r = np.mean(rs)
    
   # 3. Use the formula to calculate Cronbach's Alpha 
    cronbach_alpha = round(((N * mean_r) / (1 + (N - 1) * mean_r)), 2)
    #print ("(Cronbach's alpha = " + str(cronbach_alpha) + ", N = " + str(df.shape[0]) + ")")
    return cronbach_alpha


result_lines_to_export = []


# All Qualtrics Data (1. Sense of Presence, 2. Component-Based Usability Questions, 3. Additional Usability Questions).
# Necessary Input File: Raw Data/Qualtrics Data/qualtrics_responses.csv
# Output File: Data/sop_data.csv

qualtrics_df = pd.read_csv("Raw Data/Qualtrics Data/qualtrics_responses.csv")

# Drop the Question Text row, and reset the table index.
qualtrics_df = qualtrics_df.drop(index=0)
qualtrics_df = qualtrics_df.reset_index(drop=True)


# 1. Igroup Sense of Presence Questionnaire (http://www.igroup.org/pq/ipq/index.php)
dfSoP = qualtrics_df.filter(['Q2.1', 'Q2.2', 'Q2.3', 'Q2.4', 'Q2.5', 'Q2.6', 'Q2.7', 'Q2.8', 'Q2.9', 'Q2.10', 'Q2.11', 'Q2.12', 'Q2.13', 'Q2.14'], axis=1).astype('float64')

# Scale should be (0, 6) according to the Igroup guidelines, but qualtrics exported the data as (1,7).
dfSoP = dfSoP - 1

# Reversing the scores of the 3 questions below because of how they are worded, according to the Igroup guidelines. 
dfSoP["Q2.2"] = dfSoP["Q2.2"] * -1 + 6
dfSoP["Q2.11"] = dfSoP["Q2.11"] * -1 + 6
dfSoP["Q2.13"] = dfSoP["Q2.13"] * -1 + 6

SoP_Chronbach = cronbach_alpha(dfSoP)
result_lines_to_export.append("Cronbach's alpha:\n Sense of Presence: " + "(Cronbach's alpha = " + str(SoP_Chronbach) + ", N = " + str(dfSoP.shape[0]) + ")")

# Getting the means of scores in the same subscales.
# PRES = General Presence, SP = Spatial Presence, INV = Involvement, REAL = Experienced Realism.
dfSoP_Scores = pd.DataFrame()
dfSoP_Scores["PRES_Score"] = dfSoP["Q2.8"]
dfSoP_Scores["SP_Score"] = dfSoP[['Q2.3', 'Q2.6', 'Q2.9', 'Q2.10', 'Q2.13']].mean(axis="columns")
dfSoP_Scores["INV_Score"] = dfSoP[['Q2.1', 'Q2.7', 'Q2.11', 'Q2.14']].mean(axis="columns")
dfSoP_Scores["REAL_Score"] = dfSoP[['Q2.2', 'Q2.4', 'Q2.5', 'Q2.12']].mean(axis="columns")

dfSoP_Scores.to_csv("Data/sop_data.csv", index = "false")


# 2. Component-Based Usability Questionnaire (Brinkman, W.P., Haakma, R., and Bouwhuis, D.G. (2009). Theoretical foundation and validity of a component-based usability questionnaire, Behaviour and Information Technology, 28(2), 121 - 137.).
# Necessary Input File: Raw Data/Qualtrics Data/qualtrics_responses.csv
# Output File: Data/cbuq_data.csv
dfCBUQ = qualtrics_df.filter(['Q3.1_1', 'Q3.1_2', 'Q3.1_3', 'Q3.1_4', 'Q3.1_5', 'Q3.1_6'], axis=1).astype('float64')

CBUQ_Chronbach = cronbach_alpha(dfCBUQ)
result_lines_to_export.append(" Component-Based Usability Questions: " + "(Cronbach's alpha = " + str(CBUQ_Chronbach) + ", N = " + str(dfCBUQ.shape[0]) + ")\n\n")
print("Component-Based Usability Questions: " + "(Cronbach's alpha = " + str(CBUQ_Chronbach) + ", N = " + str(dfCBUQ.shape[0]) +")")

dfCBUQ["mean_score"] = dfCBUQ.mean(axis="columns")

dfCBUQ_Scores = dfCBUQ["mean_score"]

dfCBUQ_Scores.to_csv("Data/cbuq_data.csv", index = "false")


# 3. Additional Usability Questions.
# Necessary Input File: Raw Data/Qualtrics Data/qualtrics_responses.csv
# Output Files: Data/auq_data.csv
#               Data/auq_scores_data.csv
dfAUQ = qualtrics_df.filter(['Q4.1_1', 'Q4.1_2',
       'Q4.2_1', 'Q4.2_2', 'Q4.3_1', 'Q4.3_2', 'Q4.4_1', 'Q4.4_2', 'Q4.5_1', 'Q4.5_2',
       'Q4.6_1', 'Q4.6_2', 'Q4.7_1', 'Q4.7_2', 'Q4.8_1', 'Q4.8_2', "PID"], axis=1)

# Additional Usability Questions with Only Numerical Ratings.
dfAUQ_Scores = dfAUQ.filter(['Q4.1_1','Q4.2_1', 'Q4.3_1', 'Q4.4_1', 'Q4.5_1', 'Q4.6_1', 'Q4.7_1', 'Q4.8_1'], axis=1).astype('float64')
dfAUQ_Scores = dfAUQ_Scores.rename(columns={"Q4.1_1": "Q1", "Q4.2_1": "Q2", "Q4.3_1": "Q3", "Q4.4_1": "Q4", "Q4.5_1": "Q5", "Q4.6_1": "Q6", "Q4.7_1": "Q7", "Q4.8_1": "Q8"})

# Reversing the score of Q6 because -5 represents what we're doing right now (virtual interface moves with camera), while 5 represents the main alternative (virtual interface doesn't move).
dfAUQ_Scores["Q6"] = dfAUQ_Scores["Q6"] * -1

# Normalizing values of Q2 and 7 because 0 is the preferred value. Changing the scale from (-5,5) to be (0,5). Both -5 and 5 are deemed as "inappropriate" and 0 is deemed as "appropriate".
for key in dfAUQ_Scores["Q2"].keys():
    if dfAUQ_Scores["Q2"][key] > 0:
        dfAUQ_Scores["Q2"][key] = dfAUQ_Scores["Q2"][key] * -1
    dfAUQ_Scores["Q2"][key]+= 5 
     
for key in dfAUQ_Scores["Q7"].keys():
    if dfAUQ_Scores["Q7"][key] > 0:
        dfAUQ_Scores["Q7"][key] = dfAUQ_Scores["Q7"][key] * -1
    dfAUQ_Scores["Q7"][key]+= 5

# Additional Usability Questions (unmodified scores and open question text).
dfAUQ.to_csv("Data/auq_data.csv", index = "false")

# Additional Usability Scores.
dfAUQ_Scores.to_csv("Data/auq_scores_data.csv", index = "false")


# Igroup SoP database data to compare analysis results against the mean reported values (norm). 
# Necessary Input File: Raw Data/IPQ Database Data/ipq_data.sav
# Output File: Data/ipq_DB_means.csv
ipq_DB_df = pd.read_spss('Raw Data/IPQ Database Data/ipq_data.sav')

ipq_DB_scores_df = ipq_DB_df.filter(['g1','sp1', 'sp2', 'sp3', 'sp4', 'sp5', 'inv1', 'inv2', 'inv3', 'inv4', 'real1', 'real2', 'real3', 'real4'], axis=1).astype('float64')

ipq_DB_score_subscale_means = pd.DataFrame()
ipq_DB_score_subscale_means["pres"] = ipq_DB_scores_df['g1']
ipq_DB_score_subscale_means["sp"] = ipq_DB_scores_df[['sp1', 'sp2', 'sp3', 'sp4', 'sp5']].mean(axis="columns")
ipq_DB_score_subscale_means["inv"] = ipq_DB_scores_df[['inv1', 'inv2', 'inv3', 'inv4']].mean(axis="columns")
ipq_DB_score_subscale_means["real"] = ipq_DB_scores_df[['real2', 'real3', 'real4']].mean(axis="columns")                    

ipq_DB_score_means = round(ipq_DB_score_subscale_means.mean(axis=0),2)

result_lines_to_export.append("IGROUP Presence Questionnaire Database Norm Means:\n General Presence (PRES):" + str(ipq_DB_score_means["pres"]) + "\n Spatial Presence (SP): " + str(ipq_DB_score_means["sp"]) + "\n Involvement (INV): " + str(ipq_DB_score_means["inv"]) + "\n Experienced Realism (REAL): " + str(ipq_DB_score_means["real"]))

ipq_DB_score_means.to_csv("Data/ipq_DB_means.csv", index = "false")


# VR System Data (Experience Familiarity).
# Necessary Input Files: Raw Data/VR System Data/[N]_responses.csv
# Output File: Data/experience_familiarity_data.csv
VR_System_data_filenames = [i for i in glob.glob('Raw Data/VR System Data\*.{}'.format("csv"))]

exp_fam_dict_base = {}

responses_count = 0

for file_name in VR_System_data_filenames:
    data_file = csv.DictReader(open(file_name))
    dict_from_csv = list(data_file)

    current_response_dict = {}
    
    for instance in dict_from_csv:
        # No need for the baseline environment in the experience familiarity analysis.
        if "Baseline" not in instance["env_ID"]:
            # Craving_Q_13 is the experience familiarity question for smoker participants.
            if " Craving_Q_13" not in instance:
                current_response_dict[instance["env_ID"]] = float(instance[" Craving_Q_1"])
            else:
                current_response_dict[instance["env_ID"]] = float(instance[" Craving_Q_13"])

    exp_fam_dict_base[responses_count] = current_response_dict
    responses_count+= 1

exp_fam_dict_enhanced = {}

dict_index = 0

for user in exp_fam_dict_base:
    for env in exp_fam_dict_base[user]:
        pers = 1
        if "NOP" in env:
            pers = 0        
        env_name = env
        if "_" in env_name:
            env_name = env_name.split(str(env_name.index("_")))
        
        exp_fam_dict_enhanced[dict_index] = {        
            "subject":user,
            "env_pers":pers,
            "env_name":env_name,
            "experience_familiarity": exp_fam_dict_base[user][env],
            "sop_pres":dfSoP_Scores.iloc[user]["PRES_Score"],
            "sop_sp":dfSoP_Scores.iloc[user]["SP_Score"],
            "sop_inv":dfSoP_Scores.iloc[user]["INV_Score"],
            "sop_real":dfSoP_Scores.iloc[user]["REAL_Score"]            
        } 
        dict_index = dict_index + 1    

exp_fam_df = pd.DataFrame.from_dict(exp_fam_dict_enhanced, orient = "index")

exp_fam_df.to_csv("Data/experience_familiarity_data.csv", index = "false")


# Output Cronbach's alpha results and IGROUP Presence questionnaire norms to txt file.
with open('results_output.txt', 'w') as f:    
    f.write('\n'.join(result_lines_to_export))

